/* pop3check.c v0.30
   author: steven radack */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include <stdlib.h>

#define VERSION "0.30"

void usage()
{
	printf("usage: pop3check [options]\n");
	printf("\t-server mailserver  mailserver to connect to\n");
	printf("\t-user username      username to login with\n");
	printf("\t-pass password      password to login with\n");
	printf("\t-port port          port of pop3 server (default=110)\n");
	printf("\t-headers            show header summaries\n");
	printf("\t-v                  output server/client conversation\n");
	printf("\t-help               print out usage\n");
	printf("\t-version            print out version number\n");
	exit(0);
}

int getnword(char *s, int n, char *word)
{               
	char *token;
	token = strtok(s, " ");
	n--;
	while (n) {
		token = strtok(NULL, " ");
		n--;
	}
	if (token) {
		strcpy(word, token);
		return(1);
	}
	return(0);
}

int pop_connect(int sockfd, char *mailserver, int port)
{
	struct sockaddr_in addr;
	struct hostent *he = gethostbyname(mailserver);

	if (!he) {
		herror(mailserver);
		return(-1);
	}

	memset((char *)&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	memcpy( &(addr.sin_addr.s_addr), he->h_addr, he->h_length);
	addr.sin_port = htons(port);

	if (connect(sockfd, (struct sockaddr *)&addr, sizeof(struct sockaddr)) < 0) {
		perror("connect()");
		return(-1);
	}
	return(1);
}

void error_quit(char *err_string)
{
	printf("error: %s\n", err_string);
	exit(-1);
}

int search_header(char *header, char *string, char *info)
{
	char *position;
	int j, len;

	len = strlen(header);		
	position = strstr(header, string);
	if (position) {
		position += (strlen(string) + 1);
		for (j = 0; *position != '\n'; j++) {
			info[j] = *position;
			position++;
		}
		info[j] = '\0';
		return(1);
	}
	return(0);
}

void get_header(int sockfd, char *header, int size)
{
	int bytes = 0, pos = 0;

	for(;;) {
		bytes = read(sockfd, (header+pos), (size-bytes));
		pos += bytes;
		if (header[pos-3] == '.') {
			header[pos-1] = '\0';
			break;
		}		
	}
}

void verbose_write(char *pretext, char *text)
{
	int j, len, flag = 1;
	len = strlen(text);
	for (j = 0; j < len; j++) {
		if (flag) {
			printf("%s", pretext);
			flag = 0;
		}
		printf("%c", text[j]);
		if (text[j] == '\n') {
			flag = 1;
		}
	}
}

int main(int argc, char *argv[])
{
	char cmd[32];
	char buf[1024];
	char token[16];
	char username[32] = "";
	char password[32] = "";
	char mailserver[40] = "";
	char value[40];
	char header[1024];
	char info[64];
	int	j;
	int	sockfd;
	int	bytes = 0;
	int port = 110;
	int num_messages;
	int show_headers = 0;
	int verbose = 0;
	FILE *fp;

	if (fp != NULL) {
		fp = fopen("/home/cirin/.pop3checkrc","r");
		while (fgets(buf, sizeof(buf), fp)) {
			if (strlen(buf) == 0 || buf[0] == '\n' || buf[0] == '#') {
				continue;
			}
			sscanf(buf, "%s\t%s\n", token, value);
			if (!strcmp(token, "user")) {
				strcpy(username, value);
			} else if (!strcmp(token, "pass")) {
				strcpy(password, value);
			} else if (!strcmp(token, "server")) {
				strcpy(mailserver, value);
			} else if (!strcmp(token, "headers")) {
				if (!strcmp(value, "true")) {
					show_headers = 1;
				}
			} else {
				error_quit("invalid directive in config file");
			}	
		}
		fclose(fp);
	}
   	  
	for (j = 1; j < argc; j++) {
		if (!strcmp(argv[j], "-server") && (argc - j > 1)) {
			strncpy(mailserver,argv[++j],sizeof(mailserver));
		} else if (!strcmp(argv[j], "-user") && (argc - j > 1)) {
			strncpy(username,argv[++j],sizeof(username));
		} else if (!strcmp(argv[j], "-pass") && (argc - j > 1)) {
			strncpy(password,argv[++j],sizeof(password));
		} else if (!strcmp(argv[j], "-port") && (argc - j > 1)) {
			port = atoi(argv[++j]);
		} else if (!strcmp(argv[j], "-headers")) {
			show_headers = 1;
		} else if (!strcmp(argv[j], "-help")) {
			usage();
		} else if (!strcmp(argv[j],"-version")) {
			printf("%s\n", VERSION);
			exit(0);
		} else if (!strcmp(argv[j], "-v")) {
			verbose = 1;	
		} else {
			error_quit("invalid argument");	
		}
	}

	if (strlen(mailserver) < 1) {
		error_quit("no mailserver specified");
	}
	else if (strlen(username) < 1) {
		error_quit("no username specified");
	}
	else if (strlen(password) < 1) {
		error_quit("no password specified");
	}

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("socket()");
		exit(-1);
	}

	if (pop_connect(sockfd, mailserver, port) < 0) {
		exit(-1);
	}
	if (verbose) {
		printf("connected to %s:%d.\n", mailserver, port);
	}

	bytes = read(sockfd, buf, sizeof(buf));
	buf[bytes] = 0;
	if (verbose) {
		printf("> %s", buf);
	}

	snprintf(cmd, sizeof(cmd), "USER %s\r\n",username);
	write(sockfd, cmd, strlen(cmd));
	if (verbose) {
		printf("< %s", cmd);
	}
	bytes = read(sockfd,buf,sizeof(buf));
	buf[bytes] = 0;
	if (verbose) {
		printf("> %s", buf);
	}

	snprintf(cmd, sizeof(cmd), "PASS %s\r\n",password);
	write(sockfd, cmd, strlen(cmd));
	if (verbose) {
		printf("< %s", cmd);
	}
	bytes = read(sockfd, buf, sizeof(buf));
	buf[bytes] = 0;
	if (verbose) {
		printf("> %s", buf);
	}

	if (buf[0] == '-') {
		error_quit("username/password is incorrect");
	}

	snprintf(cmd, sizeof(cmd), "STAT\r\n");
	write(sockfd, cmd, strlen(cmd));
	if (verbose) {
		printf("< %s", cmd);
	}
	bytes = read(sockfd, buf, sizeof(buf));
	buf[bytes] = 0;
	if (verbose) {
		printf("> %s", buf);
	}

	getnword(buf, 2, token);
	num_messages = atoi(token);
	printf("Messages: %d\n", num_messages);

	if (show_headers) {
		for (j = 1; j <= num_messages; j++) {
			printf("\n");
			snprintf(cmd, sizeof(cmd), "TOP %d 0\r\n", j);
			write(sockfd, cmd, strlen(cmd) );
			get_header(sockfd, header, sizeof(header));
			if (verbose) {
				verbose_write("> ", header);
			}
			search_header(header, "From:", info);
			printf("From: %s\n", info);
			search_header(header, "Date:", info);
			printf("Date: %s\n", info);
			search_header(header, "Subject:", info);
			printf("Subject: %s\n", info);
		}
	}

	snprintf(cmd, sizeof(cmd), "QUIT");
	write(sockfd, cmd, strlen(cmd));
	if (verbose) {
		printf("< %s\n", cmd);
	}
	close(sockfd);

	return(num_messages);
}
